/***************************************************************************
 *   Copyright (C) 2008 by Marcel Hasler                                   *
 *   mahasler@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "configurationdialog.h"
#include "defs.h"
#include "kosdglobal.h"

#include <qslider.h>
#include <qspinbox.h>

#include <kcolorbutton.h>
#include <kconfig.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <klocale.h>

#include <qlabel.h>
#include <qlineedit.h>
#include <qlayout.h>
#include <qslider.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qradiobutton.h>

ConfigurationDialog::ConfigurationDialog(QWidget *parent, const char *name, WFlags f)
	: KDialogBase(IconList, i18n("Preferences"), Default|User1|Ok|Apply|Cancel, Ok, parent, name, f)
{
	// Add appearance options page
	QFrame *frame = addPage(i18n("Appearance"), i18n("Appearance"), KGlobal::iconLoader()->loadIcon("looknfeel",KIcon::Panel,0,false));
	QVBoxLayout *frameLayout = new QVBoxLayout(frame, 0, 0);
	m_configAppearancePage = new ConfigAppearance(frame);
	frameLayout->addWidget(m_configAppearancePage);

	// Add behavior options page
	frame = addPage(i18n("Behavior"), i18n("Behavior"), KGlobal::iconLoader()->loadIcon("configure",KIcon::Panel,0,false));
	frameLayout = new QVBoxLayout(frame, 0, 0);
	m_configBehaviorPage = new ConfigBehavior(frame);
	frameLayout->addWidget(m_configBehaviorPage);

	// Connect slots for ok/apply/cancel
	connect(this, SIGNAL(defaultClicked()), this, SLOT(slotSetDefaults()));
	connect(this, SIGNAL(applyClicked()), this, SLOT(slotSave()));
	connect(this, SIGNAL(okClicked()), this, SLOT(slotSave()));

	// Connect dialog slots
	connect(m_configBehaviorPage, SIGNAL(buttonShortcutsClicked()), this, SLOT(slotButtonShortcutsClicked()));

	this->enableButtonSeparator(true);
	this->setButtonGuiItem(User1, KGuiItem("About", KGlobal::iconLoader()->loadIcon("kosd",KIcon::Small,0,false)));
	this->resize(frameLayout->minimumSize());

	m_iconThemeManager = KOSDGlobal::iconThemeManager();
}


ConfigurationDialog::~ConfigurationDialog()
{
}


void ConfigurationDialog::show()
{
	this->setValues();
	this->QWidget::show();
}


void ConfigurationDialog::slotClose()
{
	emit closed();
	close();
}


void ConfigurationDialog::slotUser1()
{
	emit aboutClicked();
}


void ConfigurationDialog::setValues()
{
	KConfig *config = KGlobal::config();
	config->setGroup("Appearance");

	m_configAppearancePage->m_sliderHPos->setValue(config->readNumEntry("HPos"));
	m_configAppearancePage->m_spinboxHPos->setValue(config->readNumEntry("HPos"));

	m_configAppearancePage->m_sliderVPos->setValue(config->readNumEntry("VPos"));
	m_configAppearancePage->m_spinboxVPos->setValue(config->readNumEntry("VPos"));

	m_configAppearancePage->m_sliderSize->setValue(config->readNumEntry("Size"));
	m_configAppearancePage->m_spinboxSize->setValue(config->readNumEntry("Size"));

	m_configAppearancePage->m_buttonBackColor->setColor(config->readColorEntry("BackColor"));
	m_configAppearancePage->m_buttonFrontColor->setColor(config->readColorEntry("FrontColor"));

	bool translucencyEnabled = config->readBoolEntry("DrawTranslucent");
	m_configAppearancePage->m_checkboxTranslucency->setChecked(translucencyEnabled);
	m_configAppearancePage->m_labelOpacity->setEnabled(translucencyEnabled);
	m_configAppearancePage->m_sliderOpacity->setEnabled(translucencyEnabled);
	m_configAppearancePage->m_spinboxOpacity->setEnabled(translucencyEnabled);

	m_configAppearancePage->m_sliderOpacity->setValue(config->readNumEntry("Opacity"));
	m_configAppearancePage->m_spinboxOpacity->setValue(config->readNumEntry("Opacity"));

	m_iconThemeManager->findIconThemes();
	m_configAppearancePage->m_comboboxIconThemes->insertStringList(m_iconThemeManager->iconThemeList());
	m_configAppearancePage->m_comboboxIconThemes->setCurrentText(m_iconThemeManager->iconThemeName());
	m_configAppearancePage->slotIconThemeChanged(m_iconThemeManager->iconThemeName());

	config->setGroup("Behavior");
	m_configBehaviorPage->m_spinboxTimeout->setValue(config->readNumEntry("Timeout"));
	m_configBehaviorPage->m_radioPrimaryScreen->setChecked(config->readBoolEntry("LockPrimaryScreen"));
	m_configBehaviorPage->m_radioCursorScreen->setChecked(!m_configBehaviorPage->m_radioPrimaryScreen->isChecked());

	m_configBehaviorPage->m_comboboxMixerDevices->insertStringList(KOSDGlobal::mixer()->mixerDeviceList());
	m_configBehaviorPage->m_comboboxMixerDevices->setCurrentText(KOSDGlobal::mixer()->mixerDeviceName());
	m_configBehaviorPage->m_spinboxMixerChannel->setMaxValue(KOSDGlobal::mixer()->mixerDeviceChannels()-1);
	m_configBehaviorPage->m_spinboxMixerChannel->setValue(config->readNumEntry("MixerChannel"));
	m_configBehaviorPage->m_spinboxVolumeStep->setValue(config->readNumEntry("VolumeStep"));

	m_configBehaviorPage->m_maxBrightnessPath->setText(config->readEntry("MaxBrightnessPath"));
	m_configBehaviorPage->m_actualBrightnessPath->setText(config->readEntry("ActualBrightnessPath"));

	m_configBehaviorPage->m_chargeFullPath->setText(config->readEntry("ChargeFullPath"));
	m_configBehaviorPage->m_chargeNowPath->setText(config->readEntry("ChargeNowPath"));
}


void ConfigurationDialog::slotSave()
{
	KConfig *config = KGlobal::config();
	config->setGroup("Appearance");

	config->writeEntry("HPos", m_configAppearancePage->m_spinboxHPos->value());
	config->writeEntry("VPos", m_configAppearancePage->m_spinboxVPos->value());
	config->writeEntry("Size", m_configAppearancePage->m_spinboxSize->value());

	config->writeEntry("BackColor", m_configAppearancePage->m_buttonBackColor->color());
	config->writeEntry("FrontColor", m_configAppearancePage->m_buttonFrontColor->color());

	config->writeEntry("DrawTranslucent", m_configAppearancePage->m_checkboxTranslucency->isChecked());
	config->writeEntry("Opacity", m_configAppearancePage->m_sliderOpacity->value());

	config->writeEntry("IconTheme", m_configAppearancePage->m_comboboxIconThemes->currentText());

	config->setGroup("Behavior");
	config->writeEntry("Timeout", m_configBehaviorPage->m_spinboxTimeout->value());
	config->writeEntry("LockPrimaryScreen", m_configBehaviorPage->m_radioPrimaryScreen->isChecked());

	config->writeEntry("MixerDevice", m_configBehaviorPage->m_comboboxMixerDevices->currentText());
	config->writeEntry("MixerChannel", m_configBehaviorPage->m_spinboxMixerChannel->value());
	config->writeEntry("VolumeStep", m_configBehaviorPage->m_spinboxVolumeStep->value());

	config->writeEntry("MaxBrightnessPath", m_configBehaviorPage->m_maxBrightnessPath->text());
	config->writeEntry("ActualBrightnessPath", m_configBehaviorPage->m_actualBrightnessPath->text());

	config->writeEntry("ChargeFullPath", m_configBehaviorPage->m_chargeFullPath->text());
	config->writeEntry("ChargeNowPath", m_configBehaviorPage->m_chargeNowPath->text());

	config->sync();
	emit changed();
}


void ConfigurationDialog::slotSetDefaults()
{
	m_configAppearancePage->m_sliderHPos->setValue(OSD_DEFAULT_HPOS);
	m_configAppearancePage->m_spinboxHPos->setValue(OSD_DEFAULT_HPOS);

	m_configAppearancePage->m_sliderVPos->setValue(OSD_DEFAULT_VPOS);
	m_configAppearancePage->m_spinboxVPos->setValue(OSD_DEFAULT_VPOS);

	m_configAppearancePage->m_sliderSize->setValue(OSD_DEFAULT_SIZE);
	m_configAppearancePage->m_spinboxSize->setValue(OSD_DEFAULT_SIZE);

	m_configAppearancePage->m_buttonBackColor->setColor(OSD_DEFAULT_BACK_COLOR);
	m_configAppearancePage->m_buttonFrontColor->setColor(OSD_DEFAULT_FRONT_COLOR);

	m_configAppearancePage->m_checkboxTranslucency->setChecked(true);
	m_configAppearancePage->m_labelOpacity->setEnabled(true);
	m_configAppearancePage->m_sliderOpacity->setEnabled(true);
	m_configAppearancePage->m_spinboxOpacity->setEnabled(true);

	m_configAppearancePage->m_sliderOpacity->setValue(OSD_DEFAULT_OPACITY);
	m_configAppearancePage->m_spinboxOpacity->setValue(OSD_DEFAULT_OPACITY);

	m_configAppearancePage->m_comboboxIconThemes->setCurrentText(DEFAULT_ICON_THEME);
	m_configAppearancePage->slotIconThemeChanged(DEFAULT_ICON_THEME);

	m_configBehaviorPage->m_spinboxTimeout->setValue(OSD_DEFAULT_TIMEOUT);
	m_configBehaviorPage->m_radioPrimaryScreen->setChecked(true);
	m_configBehaviorPage->m_radioCursorScreen->setChecked(false);
	m_configBehaviorPage->m_spinboxMixerChannel->setValue(0);
	m_configBehaviorPage->m_spinboxVolumeStep->setValue(DEFAULT_VOLUME_STEP);

	m_configBehaviorPage->m_maxBrightnessPath->setText(DEFAULT_MAX_BRIGHTNESS_PATH);
	m_configBehaviorPage->m_actualBrightnessPath->setText(DEFAULT_ACTUAL_BRIGHTNESS_PATH);

	m_configBehaviorPage->m_chargeFullPath->setText(DEFAULT_CHARGE_FULL_PATH);
	m_configBehaviorPage->m_chargeNowPath->setText(DEFAULT_CHARGE_NOW_PATH);
}


void ConfigurationDialog::slotButtonShortcutsClicked()
{
	emit shortcutsClicked();
}


#include "configurationdialog.moc"
