/***************************************************************************
 *   Copyright (C) 2008 by Marcel Hasler                                   *
 *   mahasler@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef MIXER_H
#define MIXER_H

#include <dcopclient.h>
#include <vector>

struct MixerDevice
{
	QCString identifier;
	unsigned int index;
	QString name;
	unsigned int channels;
};

class Mixer
{
	friend class KOSDGlobal;
public:
	void refreshMixerDeviceList();
	QStringList mixerDeviceList();
	bool mixerDeviceAvailable(unsigned int index) const;
	bool mixerDeviceAvailable(QString name) const;
	bool setMixerDevice(unsigned int index);
	bool setMixerDevice(QString name);
	bool setMixerChannel(unsigned int channel);
	QString mixerDeviceName();
	unsigned int mixerDeviceChannels();

	unsigned int volume();
	bool isMuted();
	unsigned int volumeUp();
	unsigned int volumeDown();
	bool toggleMute();
	void setMute(bool mute);

	void updateConfiguration();

private:
	Mixer();
	~Mixer();

	bool initialize();

	DCOPClient *m_dcopClient;
	unsigned int m_volumeStep;
	unsigned int m_mixerChannel;
	std::vector<MixerDevice> m_mixerDevices;
	MixerDevice *m_currentMixerDevice;
	bool m_mixersAvailable;
};

#endif
