/***************************************************************************
 *   Copyright (C) 2008 by Marcel Hasler                                   *
 *   mahasler@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef OSD_H
#define OSD_H

#include <qpainter.h>
#include <qtimer.h>
#include <qimage.h>
#include <qdesktopwidget.h>

#include <kprogress.h>

enum OSDType { VolumeUp, VolumeDown, VolumeMute, VolumeUnmute, VolumeShow, BrightnessUp, BrightnessDown, BrightnessShow };

class OSD : public QWidget
{
	Q_OBJECT

public:
	OSD();
	~OSD();

	void showVolume(unsigned int volume, OSDType type);
	void showBrightness(unsigned int brightness, OSDType type);
	void showBattery(unsigned int charge);
	void showCustom(QString iconPath, QString text, unsigned int percent);

	void updateConfiguration();
	void updatePosition();

public slots:
	void slotHide();

private:
	float m_scaleFactor;

	QDesktopWidget *m_desktop;
	QPainter *m_painter;

	QImage *m_volumeImage, *m_volumeMuteImage, *m_volumeUpImage, *m_volumeDownImage;
	QImage *m_brightnessImage, *m_brightnessUpImage, *m_brightnessDownImage;
	QImage *m_batteryImage, *m_customImage;

	QImage *m_currentImage;
	unsigned int m_imageSize;
	QRect m_imageRect;

	QString m_text;
	QRect m_textRect;
	QFont m_textFont;

	KProgress *m_valueBar;

	QTimer *m_timer;
	QColor *m_backColor, *m_frontColor;

	unsigned int m_hPos, m_vPos;
	unsigned int m_xPos, m_yPos;
	unsigned int m_width, m_height;
	unsigned int m_timeout;

	unsigned int m_opacity;
	bool m_isTranslucent;
	bool m_isShown;
	bool m_lockPrimaryScreen;

	QPixmap *m_screenshot;

	void makeNewScreenshot();
	void paintEvent(QPaintEvent*);
};

#endif
